import stgame
import pygame
import os
import logging
from spriteutil import load_image
from sprites.timer import TimerSprite
from sprites.buttons import *
from sprites.race import *
import olpcgames.mesh as mesh
import sugar.profile
from state import FlashcardGameState
from sprites.chalkboard import *
from pygame.locals import *
class MathRaceGame(stgame.STGame):
    def __init__(self):
        super(MathRaceGame, self).__init__()
        self.timer_sprite = TimerSprite(self)
        sprite_list = [self.timer_sprite]
        self.number_sprites = [NumberInputButtonSprite(self, n) for n in range(10)]
        self.next_track_offset = 1
        self.handles_to_tracks = {}
        self.ok_button_sprite = OKButtonSprite(self)
        self.skip_button_sprite = SkipButtonSprite(self)
        self.clear_button_sprite = ClearButtonSprite(self)
        self.chalkboard_sprite = ChalkboardSprite(self)
        colors_offsets = (('red',0), ('green', 1), ('blue',2), ('yellow',3))
        self.race_tracks = [RaceTrackSprite(self, d[0], d[1]) for d in colors_offsets]

        sprite_list += self.number_sprites
        sprite_list.append(self.ok_button_sprite)
        sprite_list.append(self.skip_button_sprite)
        sprite_list.append(self.clear_button_sprite)
        sprite_list.append(self.chalkboard_sprite)
        sprite_list += self.race_tracks
        self.nick_name = sugar.profile.get_nick_name()
        self.race_tracks[0].set_player_name(self.nick_name)
        
        self.sprite_list = sprite_list
        self.all_sprites = pygame.sprite.RenderUpdates(sprite_list)
        
        self.participants = []
        self.state = FlashcardGameState()
        self.current_question = self.state.next_question()
        self.chalkboard_sprite.set_question(self.current_question)
        self.sound_correct = pygame.mixer.Sound(os.path.join('resources', 'vrmm.wav'))
        self.sound_wrong = pygame.mixer.Sound(os.path.join('resources', 'screech.wav'))
       
    
    def handle_event(self, event):
        stgame.STGame.handle_event(self, event)
        if event.type == pygame.KEYDOWN:
            if event.key == K_RETURN:
              self.chalkboard_sprite.check_current_answer()
            elif event.key == K_BACKSPACE:
                self.chalkboard_sprite.backspace_current_answer()
            elif event.key in (K_0, K_1, K_2, K_3, K_4, K_5, K_6, K_7, K_8, K_9):
                # these are coming back as unicode char ids so just hack
                # this for now
                n = {K_0:"0", K_1:"1", K_2:"2", K_3:"3",K_4:"4", K_5:"5", K_6:"6",
                     K_7:"7", K_8:"8", K_9:"9"}[event.key]
                self.chalkboard_sprite.add_to_current_answer(n)
                
        elif event.type == mesh.CONNECT:
            logging.warn( "got connect event" )
        elif event.type == mesh.PARTICIPANT_ADD:
            self.add_participant(event)
            
        elif event.type == mesh.MESSAGE_UNI or event.type == mesh.MESSAGE_MULTI:
            logging.warn ("got an update unicast event")
            self.update_progress_of_participant(event)
        
    
    def rename_track_with_handle(self, name, handle):
        self.handles_to_tracks[handle].set_player_name(name)
        
    def activate_track(self, name, handle):
        self.race_tracks[self.next_track_offset].set_player_name(name)
        self.handles_to_tracks[handle] = self.race_tracks[self.next_track_offset]
        self.next_track_offset += 1
        # XXX send a wakeup event
        
    def add_participant(self, event):
        if self.next_track_offset > 3: return
        logging.warn(event.handle)
        logging.warn(mesh.my_handle())
        if event.handle == mesh.my_handle():
          logging.warn("Got an add event myself, ignoring")
          return 

        
        self.activate_track(" ", event.handle)
        self.participants.append(event.handle)
        
        
        
    
    def update_progress_of_participant(self, event):
        (nick, current_score) = event.content.split("::<|>::")
        self.rename_track_with_handle(nick, event.handle)
        self.handles_to_tracks[event.handle].set_total_progress(float(current_score))
      
        
    def update_participants_of_progress(self, progress):
        if len(self.participants) < 1:
            logging.warn("no participants so not sending info")
        else:
            logging.warn("About to broadcast")
            for handle in self.participants:
              mesh.send_to(handle, "%s::<|>::%f" % (self.nick_name, progress))
            logging.warn("Done broadcasting progress")
            
    
    def skip_current_question(self):
        self.state.skip_question(self.current_question)
        self.current_question = self.state.next_question()
        self.chalkboard_sprite.set_question(self.current_question)
    
    def question_wrong(self):
        self.sound_wrong.play()
        
    def question_correct(self):
        self.sound_correct.play()
        self.state.answer_question(self.current_question)
        self.current_question = self.state.next_question()
        self.chalkboard_sprite.clear_current_answer()
        self.race_tracks[0].set_total_progress(self.state.get_progress())
        self.update_participants_of_progress(self.state.get_progress())
        if(self.current_question != None):
            self.chalkboard_sprite.set_question(self.current_question)
        else:
            self.chalkboard_sprite.show_done()
            self.timer_sprite.stop()
        
    def main_tick(self, render_full=False):
        self.all_sprites.update()
        updated_rects = self.all_sprites.draw(self.screen)
        if not render_full:
            pygame.display.update(updated_rects)
        else:
            pygame.display.flip()
        
    def get_background(self):
        background = pygame.image.load(os.path.join('resources','background.jpg'))
        background = background.convert()
        
        left_panel = load_image('panel_left.png')
        background.blit(left_panel,(0,25))
        
        right_panel = load_image('panel_right.png')
        background.blit(right_panel, (750,25))
        
        mcgraw_logo = load_image('mcgraw_hill_logo.gif')
        background.blit(mcgraw_logo, (135, 105))
        return background
    
game = MathRaceGame()
game.run()
