from spriteutil import load_image, STSprite
import pygame
from olpcgames import pangofont

class ChalkboardSprite(STSprite):
    def __init__(self, game):
        STSprite.__init__(self, game)
        
        self.question = "345 x 12"
        self.current_answer = ""
        self.correct_answer = ""
        
        (w,h) = (300, 250)
        (x,y) = (225, 125)
        
        overlay = pygame.surface.Surface((w,h))        
        
        self.image=overlay
        self.rect = self.image.get_rect()
        self.rect.topleft = (x,y)
        
        self.background_image = self.game.background.subsurface(self.rect)
        self.draw_question_and_answer()
    
    def set_question(self, question_hash):
        t = question_hash['terms']
        #if t[0] < 10 and t[1] < 10 and:
        if False:
          self.question = "%d %s %s" % (t[0], question_hash["operand"], t[1])
        else:
          self.question = {
            'top':"%d" % t[0],
            'operand':question_hash['operand'],
            'bottom':"%d" % t[1]
          }
        
        self.correct_answer = question_hash['answer']
        self.draw_question_and_answer()
        
    def show_done(self):
        self.question = "Congradulations!"
        self.current_answer = ""
        self.draw_question_and_answer()
        
    def check_current_answer(self):
        print "%s == %s ?" % (self.current_answer, self.correct_answer)
        if str(self.current_answer).strip() == str(self.correct_answer).strip():
            self.game.question_correct()
        else:
            self.game.question_wrong()
           
        
    def add_to_current_answer(self, new_digit):
        self.current_answer += new_digit
        self.draw_question_and_answer()
    def clear_current_answer(self):
        self.current_answer = ""
        self.draw_question_and_answer()
        
    def backspace_current_answer(self):
        self.current_answer = self.current_answer[:-1]
        self.draw_question_and_answer()
        
    def draw_question_and_answer(self):
        dest_surface = self.background_image.copy()
        font = pangofont.PangoFont( size=20)
        c = (225,225,225)
        (dw,dh) = dest_surface.get_size()
        
        if self.question:
            if type(self.question) == type(""):
              question_surface = font.render(self.question, True, c)
              (qw,qh) = question_surface.get_size()
              (qx,qy) = (int((dw - qw) / 2.0), 100)
              dest_surface.blit(question_surface, (qx,qy))
            elif type(self.question) == type({}):
              top_surface = font.render(self.question["top"], True, c)
              bottom_surface = font.render(self.question["bottom"], True, c)
              operand_surface = font.render(self.question["operand"], True, c)
              

              (top_width, top_height) = top_surface.get_size()
              (bottom_width, bottom_height) = bottom_surface.get_size()
              (operand_width, operand_height) = operand_surface.get_size()
              
              line_surface = pygame.surface.Surface((max([top_width, bottom_width]) + operand_width + 50, 5))
              line_surface.fill((255,255,255))
              
              (top_x, top_y) = (int((dw - top_width) / 2.0), 45)
              # make sure the bottom right is aligned approx with the top right
              (bottom_x, bottom_y) = (top_x + (top_width - bottom_width), 100)
              

              (operand_x, operand_y) = (min(bottom_x, top_x) - (operand_width * 1.5), 100)
              (line_x, line_y) = (operand_x - 20, 100 + operand_height + 5)
              (line_width, line_height) = line_surface.get_size()
              dest_surface.blit(top_surface, (top_x, top_y))
              dest_surface.blit(bottom_surface, (bottom_x, bottom_y))
              dest_surface.blit(operand_surface, (operand_x, operand_y))
              dest_surface.blit(line_surface, (line_x, line_y))
              
              
        if self.current_answer:
            answer_surface = font.render(self.current_answer, True, c)
            (aw,ah) = answer_surface.get_size()
            try:
              draw_at = line_y + line_height + 5
            except:
              draw_at = 180
            (ax,ay) = (int((dw - aw) / 2.0), draw_at)
            dest_surface.blit(answer_surface, (ax,ay))
        self.image = dest_surface
