from spriteutil import load_image, AlphaSprite, rounded_rect,rounded_hollow_rect
import pygame
from olpcgames import pangofont
import gradient
class RaceTrackSprite(pygame.sprite.Sprite):
    def __init__(self, game, car_color="red", offset=0):
        self.colors_to_cc = {'red':(255,0,0), 'green':(0,255,0), 'blue':(0,0,255), 'yellow':(0,255,255)}
        pygame.sprite.Sprite.__init__(self)
        self.game = game
        self.player_name = " "
        self.car_color = car_color
        
        bg_size = (self.game.background.get_width(), 50)
        bg_surface = pygame.surface.Surface(bg_size)
        bg_surface.fill((0,0,0))
        
        self.car_color = car_color
        self.color = self.colors_to_cc[car_color]
        
        inset = 2
        grad_size = (bg_size[0] - (2*inset), bg_size[1] - (2*inset))
        
        grad_surface = gradient.vertical(grad_size, (55,55,55,255), (31,31,31,255))
        
        bg_surface.blit(grad_surface, (inset, inset))
        
        r_rect = rounded_hollow_rect(self.color, (200, 40),10,4)
        bg_surface.blit(r_rect, (inset+2,inset+2))
        
        
        
        self.image_template = bg_surface
        self.image = self.image_template.copy()
        self.car_image = load_image('car_%s.png' % car_color)
        self.rect = self.image.get_rect()
        self.rect.topleft = (0, 625 + (offset * 50))
        
        self.current_progress = 0
        self.tween_progress = 0
        self.draw_car()
    
    def set_player_name(self, player_name):
        if self.player_name != " ":
          return
        self.player_name = player_name
        font = pangofont.PangoFont(size=6)
        font_surface = font.render(player_name, True, (255,255,255))
        
        
        fw,fh = font_surface.get_size()
        bw,bh = self.image_template.get_size()
        
        fx = (200-fw + 4)/2
        fy = ((bh-fh)/2)
        

        self.image_template.blit(font_surface, (fx,fy))
        self.draw_car()
        
    def set_total_progress(self, progress):
        self.current_progress = progress
    
    def update(self):
         per_tick = .005
         if self.tween_progress < self.current_progress:
             self.tween_progress += per_tick
             self.draw_car()
    
    def draw_car(self):
        if self.player_name is None: return
        progress = self.tween_progress
        self.image = self.image_template.copy()
        bw,bh = self.image_template.get_size()
        cw,ch = self.car_image.get_size()
        
        car_initial_offset = 204
        car_end_offset = bw - cw - 700
        
        car_x = car_initial_offset + (progress * (car_end_offset - car_initial_offset))
        car_y = (bh - ch) / 2
        
        
        line_w = car_x - car_initial_offset
        line_h = 10
        
        #line_surface = pygame.surface.Surface((line_w, line_h))
        #line_surface.fill(self.color)
        color_grad_start = [0,0,0,0]
        color_grad_end = [0,0,0,0]
        
        for i in range(3):
            color_grad_start[i] = color_grad_end[i] = self.color[i]
        color_grad_start[3] = 255
        color_grad_end[3] = 100
        line_surface = grad_surface = gradient.vertical((line_w,line_h), color_grad_start, color_grad_end)
        line_x = car_initial_offset
        line_y = (bh - line_h) / 2
        
        self.image.blit(line_surface,(line_x, line_y))
        self.image.blit(self.car_image, (car_x, car_y))
        
        
                                           
        
        
