from spriteutil import load_image, AlphaSprite
from olpcgames import pangofont
import pygame

class TimerSprite(AlphaSprite):
    def __init__(self, game, start_secs=0, end_secs=99 * 60):
        AlphaSprite.__init__(self, game, 'bg_timer.png', top_left = (825,75))
        timer_overlay = load_image('icon_timer.png')
        self.start_secs = start_secs
        self.end_secs = end_secs
        y = (self.shown_image.get_height() / 2) - (timer_overlay.get_height() / 2) - 2
        self.shown_image.blit(timer_overlay, (25, y))
        
        self.background_image = self.shown_image.copy()
        self.draw_time(start_secs)
        self.start_at_tick = pygame.time.get_ticks()
        self.last_secs_shown = self.start_secs
        self.stopped = False
    def stop(self):
        self.stopped = True
        
    def draw_time(self, time, color=(255,255,255)):
        minutes = str(time / 60)
        seconds = str(time % 60)
        
        if len(minutes) < 2:
           minutes = "0" + minutes
        if len(seconds) < 2:
            seconds = "0" + seconds
        
        time_string = "%s:%s" % (minutes, seconds)
        
        dest_surface = self.background_image.copy()
        font = pangofont.PangoFont( size = 12)
        font_surface = font.render(time_string, True, color)
        fw, fh = font_surface.get_size()
        dw, dh = dest_surface.get_size()
        x = (dw - fw) / 2
        y = (dh - fh) / 2
        dest_surface.blit(font_surface, (x,y))
        self.image = dest_surface
        
    def update(self):
        if self.stopped: return
        current_tick = pygame.time.get_ticks()
        delta = current_tick - self.start_at_tick
        delta_s = delta / 1000
        delta_s += self.start_secs
        if delta_s > self.last_secs_shown:
            self.last_secs_shown = delta_s
            self.tick(delta_s)
            
    def tick(self, seconds):
        # TODO: this would be better done with a pygame.time.set_timer, but that 
        # doesn't seem to work with the event pump mods on the XO
        if seconds >= self.end_secs:
            self.stopped = True
            self.draw_time(self.end_secs, color=(255,0,0))
        else:
            self.draw_time(seconds)
