import pygame
from pygame.locals import *
import os
from stgame import QuitErrorMessage

def _draw_rrect(surface, color, rect, corner_radius):
    oldclip = surface.get_clip()
    newclip = oldclip.clip(rect)
    surface.set_clip(newclip)
    rect = Rect(rect)    
    r = rect.inflate(-corner_radius*2, -corner_radius*2)
    for pos in (r.topleft, r.topright, r.bottomleft, r.bottomright):
        pygame.draw.circle(surface, color, pos, corner_radius, 0)
    pygame.draw.rect(surface, color, rect.inflate(-corner_radius*2,0), 0)
    pygame.draw.rect(surface, color, rect.inflate(0,-corner_radius*2), 0)
    surface.set_clip(oldclip)

def rounded_rect(color, size, corner_radius):
    """rounded_rect(color, size, corner_radius) -> Surface
    
    Generate a rectangle with round borders.
    """
    rect = Rect(0,0,size[0],size[1])
    s = pygame.Surface(size)
    s.fill((0,0,0))
    _draw_rrect(s, color, rect, corner_radius)
    s.set_colorkey((0,0,0))
    return s

def rounded_hollow_rect(color, size, corner_radius, border):
    xs,ys = size
    rect = Rect(0,0,xs,ys)
    inner = Rect(border,border,xs-border*2,ys-border*2)
    s = pygame.Surface(size, SRCALPHA, 32)
    s.fill((0,0,0,0))
    _draw_rrect(s, color, rect, corner_radius)
    _draw_rrect(s, (0,0,0,0), inner, corner_radius)
    return s.convert_alpha()
    
    
class STSprite(pygame.sprite.Sprite):
    def __init__(self, game):
        pygame.sprite.Sprite.__init__(self)
        self.game = game
    def handle_click(self):
        pass


class AlphaSprite(STSprite):
    def __init__(self, game, image, top_left=(0,0)):
        STSprite.__init__(self, game)
        
        img_overlay = load_image(image)
        self.rect = img_overlay.get_rect()
        self.rect.topleft = top_left
        
        self.hidden_image = self.game.background.subsurface(self.rect)
        self.shown_image = self.hidden_image.copy()
        self.shown_image.blit(img_overlay, (0,0))
        self.image = self.shown_image
        self.hidden = False
    
    def hide(self):
        if self.hidden: return
        self.image = self.hidden_image
        self.hidden = True
    
    def show(self):
        if not self.hidden: return
        self.image = self.shown_image
        self.hidden = False
    
    def click_action(self):
        pass
    
    def handle_click(self):
        if self.hidden:
            pass
        else:
            self.click_action()
        
def load_image(filename):
    return pygame.image.load(os.path.join('resources', filename)).convert_alpha()