import random

class FlashcardGameState(object):
    """docstring for FlashcardGame"""
    def __init__(self, questionTypes=['addition', 'subtraction','multiplication','division'], questionRange=[1,10], questionCount=20):
        super(FlashcardGameState, self).__init__()
        self.questionTypes = questionTypes
        self.questionRange = questionRange
       
        self.questionCount = questionCount
        self.typesToBuilders = {
          'addition':self.build_addition_question,
          'subtraction':self.build_subtraction_question,
          'multiplication':self.build_multiplication_question,
          'division':self.build_division_question
          }
    
        self.questions = [self.build_question(questionTypes) for i in range(self.questionCount)]
        self.total_questions = len(self.questions)
        self.solved_questions = []
        self.current_question_idx = 0
    
    def get_progress(self):
        total = float(self.total_questions)
        solved = float(len(self.solved_questions))
        return solved / total
    
    def next_question(self):
        if len(self.questions) <= 0:
            return None
        else:
            return self.questions.pop()
    

    def answer_question(self, question):
        self.solved_questions.append(question)
    
    def skip_question(self, question):
        self.questions.insert(0,question)
        
    def reset(self):
        [self.questions.append(q) for q in self.solved_questions]
        self.solved_questions = []
        
    def get_terms(self, questionRange):
        # We always want the larget term first so it is more suitable for subtraction and 
        # division problems, also make sure the second term can divide the first cleanly
        terms = [random.randint(questionRange[0], questionRange[1]) for i in range(2)]
        terms.sort()
        terms.reverse()
        return terms
      
    def build_question(self, questionTypes):
        i = random.randint(0, len(questionTypes) - 1)
        return self.typesToBuilders[questionTypes[i]](self.questionRange)
  
    def build_addition_question(self, questionRange):
        terms = self.get_terms(questionRange)
        answer = reduce(lambda x,y:x+y, terms)
        return {'terms':terms, 'answer':answer, 'operand':'+'}
  
    def build_subtraction_question(self, questionRange):
        terms = self.get_terms(questionRange)
        answer = reduce(lambda x,y:x-y, terms)
        return {'terms': terms, 'answer': answer, 'operand':'-'}
    
    def build_multiplication_question(self, questionRange):
        terms = self.get_terms(questionRange)
        answer = reduce(lambda x,y:x*y, terms)
        return {'terms':terms, 'answer':answer, 'operand':'x'}
  
    def build_division_question(self, questionRange):
        terms = self.get_terms(questionRange)
        answer = reduce(lambda x,y:x*y, terms)
        (terms[0], answer) = (answer, terms[0])
        return {'terms':terms, 'answer':answer, 'operand':'/'}