import pygame
from pygame.locals import *
import pygame.font
OLPC_MODE = False
import olpcgames 
import logging

log = logging.getLogger( 'stgame' )
log.setLevel( logging.DEBUG )
    
class QuitMessage(Exception):
    pass

class QuitErrorMessage(QuitMessage):
    pass

class STGame(object):
    def __init__(self, size=(1200,900)):
        pygame.init()
        #pygame.font.init()
        olpcgames.eventwrap.install()
        self.screen = pygame.display.set_mode(size)
        self.clock = pygame.time.Clock()
        self.all_sprites = []
        self.background = self.get_background()
    def delegate_events(self):
        
        for event in pygame.event.get():
            self.handle_event(event)
        
    
    def handle_event(self, event):
        if event.type == QUIT:
            raise QuitMessage('Quit Event')
        elif event.type == KEYDOWN and event.key == K_ESCAPE:
            raise QuitMessage('Escape Key')
        elif event.type == MOUSEBUTTONDOWN:
            x,y = pygame.mouse.get_pos()
            clicked_sprites = [sprite for sprite in self.all_sprites if sprite.rect.collidepoint(x,y)]
            for sprite in clicked_sprites:
                sprite.handle_click()
        elif event.type == pygame.ACTIVEEVENT:
            print "got an active event"
        elif event.type == pygame.VIDEORESIZE:
            print "got a video resize"
        elif event.type == pygame.VIDEOEXPOSE:
            print "got a video expose event"
        else:
            log.info( "got some other event %s", event )
                
    

    
    def quit(self):
        raise QuitMessage( 'Quit method' )
    
    def run(self):
        self.main_loop()
        
    def main_tick(self):
        pass
    
    def main_loop(self):
        
        pygame.event.pump()
        self.screen.blit(self.background,(0,0))
        pygame.display.flip()
        frame_rate = 20
        counter = 0
        while True:
            try: 
                counter += 1
                if (counter < frame_rate):
                    render_all=False
                else:
                    counter = 0
                    render_all=True
                
                t = self.clock.tick(frame_rate)
                if render_all:
                    self.screen.blit(self.background, (0,0))
                    
                self.delegate_events()
                self.main_tick(render_full=render_all)
                
            except QuitMessage, err:
                log.warn( "got quit message: %s", err )
                break
        log.info( 'Exiting mainloop normally' )
    
    def get_background(self):
        background = pygame.Surface(self.screen.get_size())
        background = background.convert()
        background.fill((255,0,0))
        return background
